#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for gzip compressed stream file."""

# Note: do not rename file to gzip.py this can cause the exception:
# AttributeError: 'module' object has no attribute 'GzipFile'
# when using pip.

from __future__ import unicode_literals

import unittest


# TODO: add tests for _GzipDecompressorState
# TODO: add tests for GzipMember


if __name__ == '__main__':
  unittest.main()
